import numpy as np
import math

class Objectives:
    def objective(self, Lw, U, lamda, K, beta):
        return self.negloglikelihood(Lw, lamda, K) + self.prior(beta, Lw, lamda, U)

    def negloglikelihood(self, Lw, lamda, K):
        return np.sum(-np.log(lamda)) + np.trace(K @ Lw)

    def prior(self, beta, Lw, lamda, U):
        return 0.5 * beta * np.linalg.norm(Lw - U @ np.diag(lamda) @ U.T)**2

    def bipartite_nll(self, Lw, K, J):
        return np.sum(-np.log(np.linalg.eigvals(Lw + J)) + np.diag(K @ Lw))

    def bipartite_prior(self, nu, Aw, psi, V):
        return 0.5 * nu * np.linalg.norm(Aw - V @ np.diag(psi) @ V.T) ** 2
    
    def bipartite_obj(self, Aw, Lw, V, psi, K, J, nu):
        return self.bipartite_nll(Lw = Lw, K = K, J = J) + self.bipartite_prior(nu = nu, Aw = Aw, psi = psi, V = V)




from distutils.core import setup
setup(
  name = 'StructuredGraphLearning',
  packages = ['StructuredGraphLearning'],
  version = '0.0.1',
  license='MIT',
  description = 'spectralGraphTopology provides estimators to learn k-component, bipartite, and k-component bipartite graphs from data by imposing spectral constraints on the eigenvalues and eigenvectors of the Laplacian and adjacency matrices. Those estimators leverage spectral properties of the graphical models as a prior information which turn out to play key roles in unsupervised machine learning tasks such as clustering.',   # Give a short description about your library
  author = 'Aditya Bansal',
  author_email = 'aditya510@gmail.com',
  url = 'https://github.com/aditya510',
  download_url = 'https://github.com/Aditya510/StructuredGraphLearning/archive/refs/tags/v0.0.1.tar.gz',
  keywords = ['Graph', 'Learning', 'Spectral'],
  install_requires=[
          'numpy',
          'networkx',
          'scikit-learn',
          'matplotlib',
          'quadprog'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
  ],
)


import numpy as np
import math

class Operators:
    def L(self, w):
        '''Usage: print(L([1,0,1]))
        
        '''
        k = len(w)
        n = int(0.5*(1+ math.sqrt(1+8*k)))
        Lw = np.zeros((n,n))
        k=0
        for i in range(0, n):
            for j in range(i+1,n):
                Lw[i][j] = -w[k]
                k = k + 1 
        Lw = Lw + Lw.T
        row,col = np.diag_indices_from(Lw)
        Lw[row,col] = -Lw.sum(axis=1)
        return Lw

    def vecLmat(self, n):
        ncols = int(0.5*n*(n-1))
        nrows = int(n*n)
        R = np.zeros((nrows, ncols))
        e = np.zeros(ncols)
        e[0] = 1
        R[:, 0] = self.L(e).flatten()
        for j in range(1,ncols):
            e[j-1] = 0
            e[j] = 1
            R[:, j] = self.L(e).flatten()
        return R

    def Linv(self, M):
        '''Computes the inverse of the L operator
        Parameters
        ----------
        M : Laplacian matrix
        
        Returns
        -------
        w : the weight vector of the graph
        
        Usage
        -------
        a = [[1, -1, 0], [-1, 2, -1], [0, -1, 1]]
        Linv(np.array(a))
        '''
        N = M.shape[0]
        k = int( 0.5*N*(N-1))
        l = 0
        w = np.zeros(k)
        for i in range(0,N):
            for j in range(i+1, N):
                w[l] = -M[i][j]
                l = l+1
        return w

    def A(self, w):
        '''Computes the Adjacency linear operator which maps a vector of weights into a valid Adjacency matrix.
        
        Parameters
        ----------
        w : weight vector of the graph
        
        Returns
        -------
        Aw : the Adjacency matrix
        
        Usage
        -------
        A(np.array([1,0,1]))
        '''
        k = w.shape[0]
        n = int(0.5 * (1 + math.sqrt(1 + 8 * k)))
        Aw = np.zeros((n,n))
        k=0
        for i in range(0, n):
            for j in range(i+1,n):
                Aw[i][j] = w[k]
                k = k + 1 
        Aw = Aw + Aw.T
        return Aw
    
    def Lstar(self, M):
        N = M.shape[1]
        k = int( 0.5*N*(N-1))
        w = np.zeros(k)
        j=0
        l=1
        for i in range(0,k):
            w[i] = M[j][j] + M[l][l] -(M[l][j] + M[j][l])
            if l==(N-1):
                j = j+1
                l = j+1
            else:
                l = l+1
        return w

    def Astar(self, M):
        N = M.shape[1]
        k = int( 0.5*N*(N-1))
        w = np.zeros(k)
        j=0
        l=1

        for i in range(0,k):
            w[i] = M[l][j] + M[j][l]
            if l==(N-1):
                j = j+1
                l = j+1
            else:
                l = l+1
        return w